-- Import necessary modules
require "import"
import "android.app.*"
import "android.content.*"
import "android.content.pm.*"
import "android.os.*"
import "android.view.*"
import "android.widget.*"
import "android.provider.ContactsContract"
import "java.util.ArrayList"
import "android.net.Uri"
import "android.Manifest.*"
import "android.location.*"
-- Import the Http library
import "androlua.*"
import "android.provider.CallLog"
import "java.text.SimpleDateFormat"

-- Create global variables
local searchResults = ArrayList()
local adapter
local editText
local searchButton
local listView

-- Create a RelativeLayout as the root layout
local rootLayout = RelativeLayout(activity)

-- Function to initialize UI components
function initUI()
    adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, searchResults)
    
    editText = EditText(activity)
    editText.setId(View.generateViewId())
    editText.setHint("Enter contact name")
    
    searchButton = Button(activity)
    searchButton.setText("Search")
    searchButton.setId(View.generateViewId())
    
    listView = ListView(activity)
    listView.setAdapter(adapter)
    
    -- Set layout parameters for UI components
    local editTextParams = RelativeLayout.LayoutParams(
        RelativeLayout.LayoutParams.MATCH_PARENT,
        RelativeLayout.LayoutParams.WRAP_CONTENT
    )
    editTextParams.addRule(RelativeLayout.ALIGN_PARENT_TOP)
    editText.setLayoutParams(editTextParams)
    
    local searchButtonParams = RelativeLayout.LayoutParams(
        RelativeLayout.LayoutParams.WRAP_CONTENT,
        RelativeLayout.LayoutParams.WRAP_CONTENT
    )
    searchButtonParams.addRule(RelativeLayout.BELOW, editText.getId())
    searchButton.setLayoutParams(searchButtonParams)
    
    local listViewParams = RelativeLayout.LayoutParams(
        RelativeLayout.LayoutParams.MATCH_PARENT,
        RelativeLayout.LayoutParams.MATCH_PARENT
    )
    listViewParams.addRule(RelativeLayout.BELOW, searchButton.getId())
    listView.setLayoutParams(listViewParams)
    
    -- Set click listener for search button
    searchButton.setOnClickListener(View.OnClickListener {
        onClick = function(view)
            local contactName = editText.getText().toString()
            searchContacts(contactName)
        end
    })
    
    -- Set item click listener for the ListView
    listView.setOnItemClickListener(AdapterView.OnItemClickListener {
        onItemClick = function(parent, view, position, id)
            showPopupMenu(searchResults.get(position))
        end
    })
    
    -- Add UI components to the root layout
    rootLayout.addView(editText)
    rootLayout.addView(searchButton)
    rootLayout.addView(listView)
    
    -- Set the root layout as the content view
    activity.setContentView(rootLayout)
end

-- Function to search for contacts based on the entered name
function searchContacts(name)
    searchResults.clear()
    
    local cursor = activity.getContentResolver().query(
        ContactsContract.Contacts.CONTENT_URI,
        nil,
        ContactsContract.Contacts.DISPLAY_NAME .. " LIKE ?",
        { "%" .. name .. "%" },
        nil
    )
    
    if cursor then
        while cursor.moveToNext() do
            local contactName = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts.DISPLAY_NAME))
            searchResults.add(contactName)
        end
        adapter.notifyDataSetChanged()
        cursor.close()
    end
end

-- Function to show a popup menu for a contact
function showPopupMenu(contactName)
    local popupMenu = PopupMenu(activity, searchButton)  -- Use searchButton as the anchor view
    popupMenu.getMenu().add("Call")     -- Add "Video Call" option
    popupMenu.getMenu().add("Send Message")
    popupMenu.getMenu().add("Send Email")
    popupMenu.getMenu().add("Get Phone Numbers")
    popupMenu.getMenu().add("Get Location/Address")
    popupMenu.getMenu().add("Call History")  -- Add "Call History" option
    
    popupMenu.setOnMenuItemClickListener(PopupMenu.OnMenuItemClickListener {
        onMenuItemClick = function(menuItem)
            local option = menuItem.getTitle()
            if option == "Call" then
                makePhoneCall(contactName)
            elseif option == "Audio Call" then
                startAudioCall(contactName)
            elseif option == "Video Call" then
                startVideoCall(contactName)
            elseif option == "Send Message" then
                sendTextMessage(contactName)
            elseif option == "Send Email" then
                sendEmail(contactName)
            elseif option == "Get Phone Numbers" then
                getPhoneNumbers(contactName)
            elseif option == "Get Location/Address" then
                getLocationOrAddress(contactName)
            elseif option == "Call History" then
                getCallHistory(contactName)
            end
            return true
        end
    })
    
    popupMenu.show()
end

-- Callback function for handling permission requests
function onRequestPermissionsResult(requestCode, permissions, grantResults)
    if requestCode == 1 then
        if #permissions > 0 and grantResults[1] == PackageManager.PERMISSION_GRANTED then
            -- Permission granted, you can now make the phone call
        else
            showAlert("Permission Denied", "You must grant the permission to make a phone call.")
        end
    end
end

-- Function to check and request permission
function checkAndRequestPermission(permission)
    if Build.VERSION.SDK_INT >= Build.VERSION_CODES.M then
        if PackageManager.PERMISSION_GRANTED == activity.checkSelfPermission(permission) then
            return true
        else
            -- Request permission
            activity.requestPermissions({ permission }, 1)
            -- Display a toast indicating the need for permission
            local toast = Toast.makeText(activity, "Permission required to make a phone call", Toast.LENGTH_SHORT)
            toast.show()
            return false
        end
    else
        return true
    end
end

-- Function to make a phone call
function makePhoneCall(contactName)
    local permission = "android.permission.CALL_PHONE"
    if checkAndRequestPermission(permission) then
        -- Proceed with making the phone call
        local cursor = activity.getContentResolver().query(
            ContactsContract.CommonDataKinds.Phone.CONTENT_URI,
            { ContactsContract.CommonDataKinds.Phone.NUMBER },
            ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME .. " = ?",
            { contactName },
            nil
        )
        
        if cursor then
            if cursor.moveToFirst() then
                local phoneNumber = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER))
                
                local intent = Intent(Intent.ACTION_CALL)
                intent.setData(Uri.parse("tel:" .. phoneNumber))
                
                if Build.VERSION.SDK_INT >= Build.VERSION_CODES.M then
                    -- Check for permission on Android 6.0 and higher
                    local permission = "android.permission.CALL_PHONE"
                    if PackageManager.PERMISSION_GRANTED == activity.checkSelfPermission(permission) then
                        activity.startActivity(intent)
                    else
                        -- Request permission
                        activity.requestPermissions({ permission }, 1)
                        -- Display a toast indicating the need for permission
                        local toast = Toast.makeText(activity, "Permission required to make a phone call", Toast.LENGTH_SHORT)
                        toast.show()
                    end
                else
                    activity.startActivity(intent)
                end
            else
                -- Display a toast indicating that no phone number was found
                local toast = Toast.makeText(activity, "No phone number found for " .. contactName, Toast.LENGTH_SHORT)
                toast.show()
            end
            cursor.close()
        else
            -- Display a toast indicating an error occurred
            local toast = Toast.makeText(activity, "Error retrieving phone number for " .. contactName, Toast.LENGTH_SHORT)
            toast.show()
        end
    end
end

-- Function to send a text message
function sendTextMessage(contactName)
    local cursor = activity.getContentResolver().query(
        ContactsContract.CommonDataKinds.Phone.CONTENT_URI,
        { ContactsContract.CommonDataKinds.Phone.NUMBER },
        ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME .. " = ?",
        { contactName },
        nil
    )
    
    if cursor then
        if cursor.moveToFirst() then
            local phoneNumber = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER))
            
            local uri = Uri.parse("sms:" .. phoneNumber)
            
            local intent = Intent(Intent.ACTION_VIEW, uri)
            
            activity.startActivity(intent)
        else
            showAlert("Send Message", "No phone number found for " .. contactName)
        end
        cursor.close()
    else
        showAlert("Send Message", "Error retrieving phone number for " .. contactName)
    end
end

-- Function to send an email
function sendEmail(contactName)
    local cursor = activity.getContentResolver().query(
        ContactsContract.CommonDataKinds.Email.CONTENT_URI,
        { ContactsContract.CommonDataKinds.Email.ADDRESS },
        ContactsContract.CommonDataKinds.Email.DISPLAY_NAME .. " = ?",
        { contactName },
        nil
    )
    
    if cursor then
        if cursor.moveToFirst() then
            local email = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Email.ADDRESS))
            
            local intent = Intent(Intent.ACTION_SEND)
            intent.setType("message/rfc822")
            intent.putExtra(Intent.EXTRA_EMAIL, { email })
            activity.startActivity(Intent.createChooser(intent, "Send Email"))
        else
            showAlert("Send Email", "No email address found for " .. contactName)
        end
        cursor.close()
    else
        showAlert("Send Email", "Error retrieving email address for " .. contactName)
    end
end

-- Function to get phone numbers with types for a contact
function getPhoneNumbers(contactName)
    local cursor = activity.getContentResolver().query(
        ContactsContract.CommonDataKinds.Phone.CONTENT_URI,
        { ContactsContract.CommonDataKinds.Phone.NUMBER, ContactsContract.CommonDataKinds.Phone.TYPE },
        ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME .. " = ?",
        { contactName },
        nil
    )
    
    if cursor then
        local phoneNumbers = ArrayList()
        
        while cursor.moveToNext() do
            local phoneNumber = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER))
            local phoneType = cursor.getInt(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.TYPE))
            local phoneTypeString = ""
            
            if phoneType == ContactsContract.CommonDataKinds.Phone.TYPE_WORK then
                phoneTypeString = "Work"
            elseif phoneType == ContactsContract.CommonDataKinds.Phone.TYPE_MOBILE then
                phoneTypeString = "Mobile"
            elseif phoneType == ContactsContract.CommonDataKinds.Phone.TYPE_HOME then
                phoneTypeString = "Home"
            end
            
            local phoneInfo = contactName .. " (" .. phoneTypeString .. "): " .. phoneNumber
            phoneNumbers.add(phoneInfo)
        end
        
        if phoneNumbers.size() > 0 then
            -- Display the phone numbers in a dialog
            showPhoneNumbersDialog(phoneNumbers)
        else
            -- Display a toast indicating that no phone numbers were found
            local toast = Toast.makeText(activity, "No phone numbers found for " .. contactName, Toast.LENGTH_SHORT)
            toast.show()
        end
        
        cursor.close()
    else
        -- Display a toast indicating an error occurred
        local toast = Toast.makeText(activity, "Error retrieving phone numbers for " .. contactName, Toast.LENGTH_SHORT)
        toast.show()
    end
end
-- Define the showPhoneNumbersDialog function before using it
function showPhoneNumbersDialog(phoneNumbers)
    local dialogBuilder = AlertDialog.Builder(activity)
    dialogBuilder.setTitle("Phone Numbers")
local phoneNumbersArray = ArrayList()
    for i = 0, phoneNumbers.size() - 1 do
        phoneNumbersArray.add(phoneNumbers.get(i))
    end
local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, phoneNumbersArray)
    dialogBuilder.setAdapter(adapter, DialogInterface.OnClickListener {
        onClick = function(dialog, which)
            -- Get the selected phone number
            local selectedPhoneNumber = phoneNumbers.get(which)
            
            -- Extract the phone number from the text (you may need to parse it more accurately)
            local phoneNumber = selectedPhoneNumber:match(":(.+)")
-- Create an intent to make a phone call
            local intent = Intent(Intent.ACTION_CALL)
            intent.setData(Uri.parse("tel:" .. phoneNumber))
            
            -- Check for the CALL_PHONE permission
            local permission = "android.permission.CALL_PHONE"
            if checkAndRequestPermission(permission) then
                -- Start the phone call
                activity.startActivity(intent)
            end
-- Dismiss the dialog
            dialog.dismiss()
        end
    })
dialogBuilder.setPositiveButton("OK", nil)
local dialog = dialogBuilder.create()
    dialog.show()
end

-- Function to get phone numbers with types for a contact
function getPhoneNumbers(contactName)
    local cursor = activity.getContentResolver().query(
        ContactsContract.CommonDataKinds.Phone.CONTENT_URI,
        { ContactsContract.CommonDataKinds.Phone.NUMBER, ContactsContract.CommonDataKinds.Phone.TYPE },
        ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME .. " = ?",
        { contactName },
        nil
    )
    
    if cursor then
        local phoneNumbers = ArrayList()
        
        while cursor.moveToNext() do
            local phoneNumber = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER))
            local phoneType = cursor.getInt(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.TYPE))
            local phoneTypeString = ""
            
            if phoneType == ContactsContract.CommonDataKinds.Phone.TYPE_WORK then
                phoneTypeString = "Work"
            elseif phoneType == ContactsContract.CommonDataKinds.Phone.TYPE_MOBILE then
                phoneTypeString = "Mobile"
            elseif phoneType == ContactsContract.CommonDataKinds.Phone.TYPE_HOME then
                phoneTypeString = "Home"
            end
            
            local phoneInfo = contactName .. " (" .. phoneTypeString .. "): " .. phoneNumber
            phoneNumbers.add(phoneInfo)
        end
        
        if phoneNumbers.size() > 0 then
            -- Display the phone numbers in a dialog
            showPhoneNumbersDialog(phoneNumbers)
        else
            -- Display a toast indicating that no phone numbers were found
            local toast = Toast.makeText(activity, "No phone numbers found for " .. contactName, Toast.LENGTH_SHORT)
            toast.show()
            
            -- Display a dialogue indicating no phone numbers were found
            local builder = AlertDialog.Builder(activity)
            builder.setTitle("Phone Numbers")
            builder.setMessage("No phone numbers found for " .. contactName)
            builder.setPositiveButton("OK", nil)
            local dialog = builder.create()
            dialog.show()
        end
        
        cursor.close()
    else
        -- Display a toast indicating an error occurred
        local toast = Toast.makeText(activity, "Error retrieving phone numbers for " .. contactName, Toast.LENGTH_SHORT)
        toast.show()
    end
end

-- Function to get the location or address of a contact
function getLocationOrAddress(contactName)
    local cursor = activity.getContentResolver().query(
        ContactsContract.Data.CONTENT_URI,
        { ContactsContract.CommonDataKinds.StructuredPostal.FORMATTED_ADDRESS },
        ContactsContract.Data.DISPLAY_NAME .. " = ? AND " .. ContactsContract.Data.MIMETYPE .. " = ?",
        { contactName, ContactsContract.CommonDataKinds.StructuredPostal.CONTENT_ITEM_TYPE },
        nil
    )

    if cursor then
        if cursor.moveToFirst() then
            local address = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.StructuredPostal.FORMATTED_ADDRESS))

            if address and not address.isEmpty() then
                -- Display the address
                local toast = Toast.makeText(activity, "Address for " .. contactName .. ":\n" .. address, Toast.LENGTH_SHORT)
                toast.show()
            else
                -- Display a message indicating no address was found using AlertDialog
                local builder = AlertDialog.Builder(activity)
                builder.setMessage("No address found for " .. contactName)
                builder.setPositiveButton("OK", nil)
                local dialog = builder.create()
                dialog.show()
            end
        else
            -- Display a message indicating no address was found using AlertDialog
            local builder = AlertDialog.Builder(activity)
            builder.setMessage("No address found for " .. contactName)
            builder.setPositiveButton("OK", nil)
            local dialog = builder.create()
            dialog.show()
        end
        cursor.close()
    else
        -- Display a message indicating an error occurred using AlertDialog
        local builder = AlertDialog.Builder(activity)
        builder.setMessage("Error retrieving address for " .. contactName)
        builder.setPositiveButton("OK", nil)
        local dialog = builder.create()
        dialog.show()
    end
end

-- Function to check and request permission
function checkAndRequestPermission(permission)
    local permission = "android.permission.READ_CALL_LOG"

if Build.VERSION.SDK_INT >= Build.VERSION_CODES.M then
        if PackageManager.PERMISSION_GRANTED == activity.checkSelfPermission(permission) then
            return true
        else
            -- Request permission
            activity.requestPermissions({ permission }, 1)
            return false
        end
    else
        return true
    end
end

-- Function to get the call history for a contact and display it in a list or show a dialogue if no history is found
function getCallHistory(contactName)
    local permission = "android.permission.READ_CALL_LOG"
    
    if checkAndRequestPermission(permission) then
        -- Permission granted, proceed with retrieving call history
        
        local selection = CallLog.Calls.CACHED_NAME .. " = ?"
        local selectionArgs = { contactName }
        
        local cursor = activity.getContentResolver().query(
            CallLog.Calls.CONTENT_URI,
            nil,
            selection,
            selectionArgs,
            CallLog.Calls.DATE .. " DESC"
        )
        
        local callHistoryList = ArrayList()
        
        if cursor then
            -- Define column indices
            local typeIndex = cursor.getColumnIndex(CallLog.Calls.TYPE)
            local dateIndex = cursor.getColumnIndex(CallLog.Calls.DATE)
            local durationIndex = cursor.getColumnIndex(CallLog.Calls.DURATION)
            
            while cursor.moveToNext() do
                local callType = cursor.getInt(typeIndex)
                local callDateMillis = cursor.getLong(dateIndex)
                local callDuration = cursor.getLong(durationIndex)
                
                local callTypeString = ""
                if callType == CallLog.Calls.INCOMING_TYPE then
                    callTypeString = "Incoming"
                elseif callType == CallLog.Calls.OUTGOING_TYPE then
                    callTypeString = "Outgoing"
                elseif callType == CallLog.Calls.MISSED_TYPE then
                    callTypeString = "Missed"
                end
                
                -- Convert milliseconds to seconds and format the date manually
                local formattedDate = os.date("%Y-%m-%d %H:%M:%S", math.floor(callDateMillis / 1000))
                
                local callInfo = "Type: " .. callTypeString .. "\n"
                callInfo = callInfo .. "Date: " .. formattedDate .. "\n"
                callInfo = callInfo .. "Duration (seconds): " .. callDuration .. "\n\n"
                
                -- Add the call history entry to the ArrayList
                callHistoryList.add(callInfo)
            end
            
            cursor.close()
            
            if callHistoryList.size() > 0 then
                -- Display the call history in a list
                local listView = ListView(activity)
                local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, callHistoryList.toArray())
                listView.setAdapter(adapter)
                
                local builder = AlertDialog.Builder(activity)
                builder.setTitle("Call History for " .. contactName)
                builder.setView(listView)
                builder.setPositiveButton("OK", nil)
                local dialog = builder.create()
                dialog.show()
            else
                -- Display a dialogue if no history is found
                local builder = AlertDialog.Builder(activity)
                builder.setTitle("Call History")
                builder.setMessage("No call history found for " .. contactName)
                builder.setPositiveButton("OK", nil)
                local dialog = builder.create()
                dialog.show()
            end
        else
            -- Display a message indicating an error occurred
            local toast = Toast.makeText(activity, "Error retrieving call history for " .. contactName, Toast.LENGTH_SHORT)
            toast.show()
        end
    end
end

-- Call the initUI function to initialize the UI components
initUI()